/******************************************************************************
 * tipper_red.h
 * Federico Bareilles
 * 31/01/2003
 *
 * 24/07/2003: Add warning element in struct *tdata;
 * 15/10/2003: Add --tsig option.
 * 21/10/2003: Add --online option (v0.1f).
 *
 *****************************************************************************/
#ifndef __TIPPER_RED_H
#define __TIPPER_RED_H


#define WARNING      1          /* WARNING message in raw data file. */
#define NU_ERROR     2          /* Numerical error occured during fiting. */
#define OBSTACLE     4          /* v_sky does not increase with Airmass */
#define SKY_G_REF    8          /* v_sky > v_ref. */
#define BADTAU      16          /* Fit tau < 0.0 */
#define VCOMPLOW    32          /* Alternate Vref_ADC used. */
#define BADANGLES   64          /* num angles < read_ang */
#define BADRAWFILE 128          /* Error un raw data */

/* setup mode : */
#define SM_STD       1          /* Standard mode */
#define SM_OLINE     2          /* On line mode for point by point */ 

/* Parameters */ 
#define MAX_ANG 8               /* Numbers of angles in raw data */
#define GAINOFF 1.0		/* Gain applied to Vsky */
#define ANGOFF  0.0  		/* Box Angle offset (degrees) */ 
#define MIROFF  2.5             /* Mirror Angle Offset (degrees) */
//#define DAYOF	0.0		/* Day Offset */
//#define HROF	0.0		/* Hour Offset */
//#define V_OFFSET 6.0		/* Offset at the ADC input in mV */

/* #define DEG_RAD 0.01745329252 */  /* 3.141592654/180.0 */
/* 3.14159265358979323846264338328/180.0 */
#define DEG_RAD   0.01745329251994329576

/* Reference day: `date --date="1992-1-1" "+%s"` */
#define REFDAY 694231200        /* reference day in secons */

#define SQR(x)  ((x)*(x))

#undef PVERB
#ifdef VERBOSE_MODE
#  define PVERB(fmt, args...) if(verbose_flag) fprintf(stderr, fmt, ## args)
#else 
#  define PVERB(fmt, args...)
#endif

struct tpoint{
  int pos;
  float airmass;
  float v_sky;
  float v_sky_sigma;
  float v_ref;
  float v_ref_sigma;
};

struct fit{
  float tau;
  float b;
  float sigma_tau;
  float sigma_b;
  float resid;
  float chi2f;
};

struct tdata{
  int unit;
  int error_flag;
  float date;
  float t_adq;
  float t_abs;
  float t_gun;
  float t_amp;
  float v_comp;
  float v_comp_sigma;
  float v_gun;
  float v_gun_sigma;
  int n;
  char *warning;
  struct tpoint points[8];
  struct fit *f;
  struct tdata *next;
};


struct zener_table{
  float *in;
  float *out;
  int size;
};


struct instrument{
  int unit;
  char *ad_file;
  float vref1_ADC;
  float vref2_ADC;
  float agrd_ADC;
  struct zener_table *zt;
};


struct setup{
  char *file_in;
  struct tdata *data;
  struct instrument *tipper;
  int read_ang;
  int sec_offset; /* for correction the coputer clock */
  float (*spillover)(float);
  struct fit *(*fit_metod) (float *xd, float *yd, 
			    float *sd, int num, float Ta, float Tl);
  float (*y_transform)(float);
  float (*sigma_propagation)(struct tpoint *p);
  int mode;
};

extern char *program_name;
extern char *program_version;

/* Flags: */
extern int verbose_flag;

void usage();
void help();

//struct tdata *get_raw_data(FILE *fp_in, struct setup *s);
int get_raw_data(FILE *fp_in, struct setup *s);
int free_tdata(struct tdata *data);

struct zener_table *get_table_zener(struct instrument *t);
int free_zener_table(struct instrument *t);
float zener_cor(float v, struct instrument *t);

struct instrument *set_instrument(int unit);
int free_instrument(struct instrument *t);

//int print_header(FILE *fp_out, struct setup *s);
int print_data(FILE *fp, struct setup *s);
int print_tempe(FILE *fp, struct setup *s);
int print_tsigma(FILE *fp, struct setup *s);
int print_data_no_red(FILE *fp_out, struct setup *s);

int data_correction(struct setup *s);
int data_check_airmass(struct setup *s);
int data_reduction(struct setup *s);

/* fit metods: */
struct fit *linear_fit(float *x, float *y, 
		       float *sigma, int num, float Ta, float Tl);
struct fit *linear_matrix_fit(float *x, float *y,
			      float *sigma, int num, float Ta, float Tl);
struct fit *no_linear_fit(float *x, float *y, 
			  float *sigma, int num, float Ta, float Tl);

#endif

